require "import"
import "android.widget.*"
import "android.view.*"
import "android.content.*"
import "android.net.Uri"
import "android.os.Environment"
import "android.graphics.pdf.PdfDocument"
import "android.graphics.Paint" -- Adicionado explicitamente para corrigir o erro
import "java.io.*"

-- Criar diretório se não existir
local function criarDiretorio(caminho)
    local dir = File(caminho)
    if not dir.exists() then
        dir.mkdirs()
    end
end

-- Lista de formatos suportados
local formatosSuportados = {".txt", ".docs", ".docx", ".pdf", ".rtf", ".odt", ".ott", ".doc", ".wps",
".sxw", ".pages", ".tex", ".log", ".md", ".mdx", ".rst", ".wri", ".adoc",
".asciidoc", ".nfo", ".me", ".1st", ".man", ".lit", ".srt", ".sub",
".texi", ".roff", ".troff", ".creole", ".markdown", ".dita", ".chm", ".ris", ".bib", ".bibtex",
".xps", ".eml", ".msg", ".xls", ".xlsx", ".xlsm", ".ods", ".fods", ".csv", ".tsv", ".numbers",
".dif", ".slk", ".ppt", ".pptx", ".odp", ".key", ".pot", ".potx",
".sxi", ".html", ".htm", ".xhtml", ".xml", ".xsl", ".json", ".yaml", ".yml", ".svg",
".webp", ".rss", ".py", ".pyc", ".pyo", ".pyw", ".ip Wb", ".java", ".class", ".jar", ".jsp",
".c", ".cpp", ".cxx", ".cc", ".h", ".hpp", ".cs", ".js", ".mjs", ".cjs", ".jsx",
".ts", ".tsx", ".php", ".phtml", ".rb", ".erb", ".go", ".rs", ".swift",
".kt", ".kts", ".sh", ".bash", ".zsh", ".fish", ".bat", ".cmd", ".pl", ".pm",
".r", ".scala", ".sql", ".lua", ".hs", ".lhs", ".jl", ".dart", ".erl", ".hrl",
".ex", ".exs", ".m", ".mm", ".f", ".f90", ".f95", ".cbl", ".cob", ".vbs",
".vb", "Makefile", ".mk", "Dockerfile", ".coffee", ".clj", ".cljs", ".v",
".vhd", ".vhdl", ".rkt", ".adb", ".ads",
".phar", ".mjs", ".gem", ".dart_tool", ".asm", ".s",
".ini", ".cfg", ".conf", ".env", ".gitignore", ".htaccess", ".editorconfig",
".crt", ".pem", ".key", ".yaml", ".json5",
".epub", ".mobi", ".azw3", ".fb2", ".opf", ".azw", ".ibooks",
".dita", ".db", ".sqlite", ".properties", ".plist", ".toml", ".bib",
".accdb", ".mdb", ".out", ".dbf", ".jsonl",
".psd", ".ai", ".fig", ".sketch", ".xd"}

-- Verificar formato do arquivo
local function ehFormatoSuportado(extensao)
    for _, ext in ipairs(formatosSuportados) do
        if ext:lower() == extensao:lower() then return true end
    end
    return false
end

-- Salvar como arquivo de texto
function salvarComoArquivoTexto(texto, caminhoArquivo)
    local sucesso, erro = pcall(function()
        local fos = FileOutputStream(caminhoArquivo)
        local escritor = OutputStreamWriter(fos)
        escritor.write(texto)
        escritor.close()
        fos.close()
        service.postSpeak(500, "Arquivo de texto criado com sucesso, salvo em: " .. caminhoArquivo)
        exibirOpcoesESair(caminhoArquivo)
    end)

    if not sucesso then
        service.postSpeak(600, "Não foi possível salvar o arquivo de texto: " .. tostring(erro))
    end
end

-- Obter extensão do nome do arquivo ou do Spinner
local function obterExtensaoArquivo(nomeArquivo, extensaoSelecionada)
    local extensaoEntrada = nomeArquivo:match("%.([%w]+)$")
    if extensaoEntrada and ehFormatoSuportado("." .. extensaoEntrada) then
        return "." .. extensaoEntrada
    else
        return extensaoSelecionada
    end
end

-- Converter texto para arquivo
function converterTextoParaArquivo(texto, nomeArquivo, extensaoSelecionada)
    local caminhoPasta = Environment.getExternalStorageDirectory().getAbsolutePath() .. "/notas/"
    criarDiretorio(caminhoPasta)

    local extensao = obterExtensaoArquivo(nomeArquivo, extensaoSelecionada)
    local nomeSemExtensao = nomeArquivo:gsub("%.[%w]+$", "")
    local caminhoArquivo = caminhoPasta .. nomeSemExtensao .. extensao

    if extensao == ".pdf" then
        salvarComoPDF(texto, caminhoArquivo)
    elseif ehFormatoSuportado(extensao) then
        salvarComoArquivoTexto(texto, caminhoArquivo)
    else
        service.postSpeak(700, "Formato não suportado.")
    end
end

-- Salvar como PDF (usando a lógica do código funcional)
function salvarComoPDF(texto, caminhoArquivo)
    local sucesso, erro = pcall(function()
        local pdfDoc = PdfDocument()
        local infoPagina = PdfDocument.PageInfo.Builder(595, 842, 1).create()
        local pagina = pdfDoc.startPage(infoPagina)
        local canvas = pagina.getCanvas()
        local pincel = Paint()
        pincel.setTextSize(12)
        pincel.setColor(0xFF000000)

        local y = 25
        local alturaLinha = 15
        local larguraMaxima = 575

        texto = sanitizarTexto(texto)
        for linha in texto:gmatch("[^\n]+") do
            if pincel.measureText(linha) > larguraMaxima then
                local palavras = {}
                for palavra in linha:gmatch("%S+") do table.insert(palavras, palavra) end

                local linhaAtual = ""
                for _, palavra in ipairs(palavras) do
                    local linhaTemporaria = linhaAtual .. (linhaAtual == "" and "" or " ") .. palavra
                    if pincel.measureText(linhaTemporaria) > larguraMaxima then
                        canvas.drawText(linhaAtual, 10, y, pincel)
                        y = y + alturaLinha
                        linhaAtual = palavra
                    else
                        linhaAtual = linhaTemporaria
                    end
                end
                canvas.drawText(linhaAtual, 10, y, pincel)
                y = y + alturaLinha
            else
                canvas.drawText(linha, 10, y, pincel)
                y = y + alturaLinha
            end
        end

        pdfDoc.finishPage(pagina)

        local fos = FileOutputStream(caminhoArquivo)
        pdfDoc.writeTo(fos)
        pdfDoc.close()
        fos.close()

        service.postSpeak(800, "Arquivo PDF criado em: " .. caminhoArquivo)
        exibirOpcoesESair(caminhoArquivo)
    end)

    if not sucesso then
        service.postSpeak(1000, "Erro ao criar arquivo PDF: " .. tostring(erro))
    end
end

-- Sanitizar texto
function sanitizarTexto(texto)
    return texto:gsub("\r\n", "\n"):gsub("\r", "\n")
end

-- Exibir opções e sair
function exibirOpcoesESair(caminhoArquivo)
    local function fecharTodosDialogos()
        if dlg then dlg.dismiss() end
        if mainDlg then mainDlg.dismiss() end
    end

    local layoutOpcoes = {
        LinearLayout,
        orientation = "vertical",
        backgroundColor = "#212121",
        padding = "16dp",
        {
            Button,
            text = "Excluir",
            textColor = "#FFFFFF",
            backgroundColor = "#D32F2F",
            layout_width = "match_parent",
            layout_height = "wrap_content",
            layout_marginBottom = "8dp",
            onClick = function()
                excluirArquivo(caminhoArquivo)
                fecharTodosDialogos()
            end
        },
        {
            Button,
            text = "Compartilhar",
            textColor = "#FFFFFF",
            backgroundColor = "#1976D2",
            layout_width = "match_parent",
            layout_height = "wrap_content",
            layout_marginBottom = "8dp",
            onClick = function()
                service.shareFile(caminhoArquivo)
                service.postSpeak(600, "Compartilhando arquivo: " .. caminhoArquivo)
                fecharTodosDialogos()
            end
        },
        {
            Button,
            text = "Sair",
            textColor = "#FFFFFF",
            backgroundColor = "#757575",
            layout_width = "match_parent",
            layout_height = "wrap_content",
            onClick = function()
                fecharTodosDialogos()
                service.postSpeak(500, "Até logo")
            end
        }
    }

    dlg = LuaDialog(service)
    dlg.setTitle("Opções para o arquivo: " .. caminhoArquivo)
    dlg.setView(loadlayout(layoutOpcoes))
    dlg.show()
end

-- Excluir arquivo
function excluirArquivo(caminhoArquivo)
    local arquivo = File(caminhoArquivo)
    if arquivo.exists() then
        arquivo.delete()
        service.postSpeak(700, "Arquivo excluído com sucesso: " .. caminhoArquivo)
    else
        service.postSpeak(800, "O arquivo não existe.")
    end
end

-- Interface do usuário modificada
function aoClicar()
    local layout = {
        LinearLayout,
        orientation = "vertical",
        layout_width = "match_parent",
        layout_height = "match_parent",
        backgroundColor = "#212121",
        padding = "16dp",
        {
            ScrollView,
            layout_width = "match_parent",
            layout_height = "0dp",
            layout_weight = "1",
            {
                LinearLayout,
                orientation = "vertical",
                layout_width = "match_parent",
                layout_height = "wrap_content",
                {
                    TextView,
                    text = "Selecione o formato do arquivo:",
                    textColor = "#FFFFFF",
                    layout_width = "wrap_content",
                    layout_height = "wrap_content",
                    layout_marginBottom = "8dp",
                },
                {
                    Spinner,
                    id = "spinnerExtensaoArquivo",
                    layout_width = "match_parent",
                    layout_marginBottom = "16dp",
                    adapter = ArrayAdapter(service, android.R.layout.simple_spinner_item, formatosSuportados),
                },
                {
                    EditText,
                    id = "nomeArquivo",
                    hint = "Digite o nome do arquivo",
                    textColor = "#FFFFFF",
                    hintTextColor = "#B0BEC5",
                    layout_width = "match_parent",
                    layout_marginBottom = "16dp",
                },
                {
                    EditText,
                    id = "textoEdicao",
                    hint = "Digite o texto",
                    textColor = "#FFFFFF",
                    hintTextColor = "#B0BEC5",
                    layout_width = "match_parent",
                    layout_height = "0dp",
                    layout_weight = "1",
                },
                {
                    Button,
                    text = "Limpar Caixa de Edição",
                    textColor = "#FFFFFF",
                    backgroundColor = "#F57C00",
                    layout_width = "match_parent",
                    layout_margin = "4dp",
                    layout_marginTop = "16dp",
                    onClick = function()
                        textoEdicao.setText("")
                        service.postSpeak(400, "Texto limpo")
                    end
                },
            },
        },
        {
            LinearLayout,
            orientation = "horizontal",
            layout_width = "match_parent",
            layout_height = "wrap_content",
            gravity = "center",
            layout_marginTop = "16dp",
            {
                Button,
                text = "Colar da Área de Transferência",
                textColor = "#FFFFFF",
                backgroundColor = "#1976D2",
                layout_width = "0dp",
                layout_weight = "1",
                layout_margin = "4dp",
                onClick = function()
                    local clipboard = service.getSystemService(Context.CLIPBOARD_SERVICE)
                    local clipData = clipboard.getPrimaryClip()
                    if clipData ~= nil and clipData.getItemCount() > 0 then
                        local textoColado = clipData.getItemAt(0).getText()
                        if textoColado then
                            textoEdicao.setText(textoColado)
                            service.postSpeak(550, "Texto colado da área de transferência")
                        else
                            service.postSpeak(650, "Nenhum texto na área de transferência")
                        end
                    else
                        service.postSpeak(750, "Área de transferência vazia")
                    end
                end
            },
            {
                Button,
                text = "Colar dos Favoritos",
                textColor = "#FFFFFF",
                backgroundColor = "#388E3C",
                layout_width = "0dp",
                layout_weight = "1",
                layout_margin = "4dp",
                onClick = function()
                    local favoritos = service.getFavoritesList()
                    if #favoritos > 0 then
                        local textoFavorito = favoritos[0]
                        textoEdicao.setText(textoFavorito)
                        service.postSpeak(700, "Texto colado dos favoritos")
                    else
                        service.postSpeak(850, "Nenhum texto nos favoritos")
                    end
                end
            },
        },
        {
            LinearLayout,
            orientation = "horizontal",
            layout_width = "match_parent",
            layout_height = "wrap_content",
            gravity = "center",
            layout_marginTop = "16dp",
            {
                Button,
                text = "Converter e Salvar Arquivo",
                textColor = "#FFFFFF",
                backgroundColor = "#0288D1",
                layout_width = "0dp",
                layout_weight = "1",
                layout_margin = "4dp",
                onClick = function()
                    local texto = textoEdicao.getText().toString()
                    local nomeArquivo = nomeArquivo.getText().toString()
                    local extensaoSelecionada = spinnerExtensaoArquivo.getSelectedItem()

                    if texto == "" then
                        service.postSpeak(400, "Por favor, digite um texto.")
                    elseif nomeArquivo == "" then
                        service.postSpeak(450, "Por favor, digite o nome do arquivo.")
                    else
                        converterTextoParaArquivo(texto, nomeArquivo, extensaoSelecionada)
                    end
                end
            },
            {
                Button,
                text = "Sair",
                textColor = "#FFFFFF",
                backgroundColor = "#D32F2F",
                layout_width = "0dp",
                layout_weight = "1",
                layout_margin = "4dp",
                onClick = function()
                    service.postSpeak(500, "Até logo")
                    if mainDlg then mainDlg.dismiss() end
                end
            },
        },
    }

    mainDlg = LuaDialog(service)
    mainDlg.setTitle("Criar Arquivo de Texto")
    mainDlg.setView(loadlayout(layout))
    mainDlg.show()
end

-- Iniciar aplicativo
aoClicar()